using System;
using System.Collections.ObjectModel;
using System.Linq;

namespace GLTFViewer.Engine;

public class Node
{
    public string Name { get; set; } = "Node";
    public ObservableCollection<Node> Children { get; } = [];
    public ObservableCollection<NodeProperty> Properties { get; } = [];
    
    public virtual void Update()
    {
        foreach (var node in Children)
        {
            node.Update();
        }
    }
    
    public virtual void EnterScene()
    {
        foreach (var node in Children)
        {
            node.EnterScene();
        }
    }
    
    public virtual void ExitScene()
    {
        foreach (var node in Children)
        {
            node.ExitScene();
        }
    }

    public void AddChild(Node child)
    {
        Children.Add(child);
    }

    public void RemoveChild(Node child)
    {
        Children.Remove(child);
    }

    public void AddProperty(NodeProperty property)
    {
        if (Properties.Any(p => p.Name == property.Name))
        {
            throw new InvalidOperationException($"A property with the name {property.Name} already exists in node {Name}.");
        }

        Properties.Add(property);
    }

    public void RemoveProperty(NodeProperty property)
    {
        if (!Properties.Remove(property))
        {
            Console.WriteLine($"Could not remove property {property.Name} from node {Name}: Property not found.");
        }
    }

    public void SetProperty<T>(string name, T value)
    {
        var property = Properties.OfType<NodeProperty<T>>().FirstOrDefault(p => p.Name == name);
        if (property == null)
        {
            property = new NodeProperty<T>(name, value);
            Properties.Add(property);
        }
        else
        {
            property.Value = value;
        }
    }
}